package fr.ippon.geohipster.web.rest;

import fr.ippon.geohipster.GeohipsterApp;

import fr.ippon.geohipster.domain.Geocity;
import fr.ippon.geohipster.repository.GeocityRepository;
import fr.ippon.geohipster.service.GeocityService;
import fr.ippon.geohipster.web.rest.errors.ExceptionTranslator;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.MockitoAnnotations;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.data.web.PageableHandlerMethodArgumentResolver;
import org.springframework.http.MediaType;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.EntityManager;
import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.hasItem;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

/**
 * Test class for the GeocityResource REST controller.
 *
 * @see GeocityResource
 */
@RunWith(SpringRunner.class)
@SpringBootTest(classes = GeohipsterApp.class)
public class GeocityResourceIntTest {

    @Autowired
    private GeocityRepository geocityRepository;

    @Autowired
    private GeocityService geocityService;

    @Autowired
    private MappingJackson2HttpMessageConverter jacksonMessageConverter;

    @Autowired
    private PageableHandlerMethodArgumentResolver pageableArgumentResolver;

    @Autowired
    private ExceptionTranslator exceptionTranslator;

    @Autowired
    private EntityManager em;

    private MockMvc restGeocityMockMvc;

    private Geocity geocity;

    @Before
    public void setup() {
        MockitoAnnotations.initMocks(this);
        GeocityResource geocityResource = new GeocityResource(geocityService);
        this.restGeocityMockMvc = MockMvcBuilders.standaloneSetup(geocityResource)
            .setCustomArgumentResolvers(pageableArgumentResolver)
            .setControllerAdvice(exceptionTranslator)
            .setMessageConverters(jacksonMessageConverter).build();
    }

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static Geocity createEntity(EntityManager em) {
        Geocity geocity = new Geocity();
        return geocity;
    }

    @Before
    public void initTest() {
        geocity = createEntity(em);
    }

    @Test
    @Transactional
    public void createGeocity() throws Exception {
        int databaseSizeBeforeCreate = geocityRepository.findAll().size();

        // Create the Geocity
        restGeocityMockMvc.perform(post("/api/geocities")
            .contentType(TestUtil.APPLICATION_JSON_UTF8)
            .content(TestUtil.convertObjectToJsonBytes(geocity)))
            .andExpect(status().isCreated());

        // Validate the Geocity in the database
        List<Geocity> geocityList = geocityRepository.findAll();
        assertThat(geocityList).hasSize(databaseSizeBeforeCreate + 1);
        Geocity testGeocity = geocityList.get(geocityList.size() - 1);
    }

    @Test
    @Transactional
    public void createGeocityWithExistingId() throws Exception {
        int databaseSizeBeforeCreate = geocityRepository.findAll().size();

        // Create the Geocity with an existing ID
        geocity.setId(1L);

        // An entity with an existing ID cannot be created, so this API call must fail
        restGeocityMockMvc.perform(post("/api/geocities")
            .contentType(TestUtil.APPLICATION_JSON_UTF8)
            .content(TestUtil.convertObjectToJsonBytes(geocity)))
            .andExpect(status().isBadRequest());

        // Validate the Alice in the database
        List<Geocity> geocityList = geocityRepository.findAll();
        assertThat(geocityList).hasSize(databaseSizeBeforeCreate);
    }

    @Test
    @Transactional
    public void getAllGeocities() throws Exception {
        // Initialize the database
        geocityRepository.saveAndFlush(geocity);

        // Get all the geocityList
        restGeocityMockMvc.perform(get("/api/geocities?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_UTF8_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(geocity.getId().intValue())));
    }

    @Test
    @Transactional
    public void getGeocity() throws Exception {
        // Initialize the database
        geocityRepository.saveAndFlush(geocity);

        // Get the geocity
        restGeocityMockMvc.perform(get("/api/geocities/{id}", geocity.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_UTF8_VALUE))
            .andExpect(jsonPath("$.id").value(geocity.getId().intValue()));
    }

    @Test
    @Transactional
    public void getNonExistingGeocity() throws Exception {
        // Get the geocity
        restGeocityMockMvc.perform(get("/api/geocities/{id}", Long.MAX_VALUE))
            .andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    public void updateGeocity() throws Exception {
        // Initialize the database
        geocityService.save(geocity);

        int databaseSizeBeforeUpdate = geocityRepository.findAll().size();

        // Update the geocity
        Geocity updatedGeocity = geocityRepository.findOne(geocity.getId());

        restGeocityMockMvc.perform(put("/api/geocities")
            .contentType(TestUtil.APPLICATION_JSON_UTF8)
            .content(TestUtil.convertObjectToJsonBytes(updatedGeocity)))
            .andExpect(status().isOk());

        // Validate the Geocity in the database
        List<Geocity> geocityList = geocityRepository.findAll();
        assertThat(geocityList).hasSize(databaseSizeBeforeUpdate);
        Geocity testGeocity = geocityList.get(geocityList.size() - 1);
    }

    @Test
    @Transactional
    public void updateNonExistingGeocity() throws Exception {
        int databaseSizeBeforeUpdate = geocityRepository.findAll().size();

        // Create the Geocity

        // If the entity doesn't have an ID, it will be created instead of just being updated
        restGeocityMockMvc.perform(put("/api/geocities")
            .contentType(TestUtil.APPLICATION_JSON_UTF8)
            .content(TestUtil.convertObjectToJsonBytes(geocity)))
            .andExpect(status().isCreated());

        // Validate the Geocity in the database
        List<Geocity> geocityList = geocityRepository.findAll();
        assertThat(geocityList).hasSize(databaseSizeBeforeUpdate + 1);
    }

    @Test
    @Transactional
    public void deleteGeocity() throws Exception {
        // Initialize the database
        geocityService.save(geocity);

        int databaseSizeBeforeDelete = geocityRepository.findAll().size();

        // Get the geocity
        restGeocityMockMvc.perform(delete("/api/geocities/{id}", geocity.getId())
            .accept(TestUtil.APPLICATION_JSON_UTF8))
            .andExpect(status().isOk());

        // Validate the database is empty
        List<Geocity> geocityList = geocityRepository.findAll();
        assertThat(geocityList).hasSize(databaseSizeBeforeDelete - 1);
    }

    @Test
    @Transactional
    public void equalsVerifier() throws Exception {
        TestUtil.equalsVerifier(Geocity.class);
        Geocity geocity1 = new Geocity();
        geocity1.setId(1L);
        Geocity geocity2 = new Geocity();
        geocity2.setId(geocity1.getId());
        assertThat(geocity1).isEqualTo(geocity2);
        geocity2.setId(2L);
        assertThat(geocity1).isNotEqualTo(geocity2);
        geocity1.setId(null);
        assertThat(geocity1).isNotEqualTo(geocity2);
    }
}
